#include <gjournal/journalfile.hpp>

namespace gjournal
{
	JournalFile::JournalFile( JournalWindow *window )
	{
		srand( time( 0 ) );

		this->window = window;
	}

	bool JournalFile::_createNewFile( std::string file, std::string &error )
	{
		sqlitem::SQliteManager db;
		if( !db.connect( file ) )
		{
			error = db.lastError();
			return false;
		}

		std::string queries[3];
		queries[0] = "CREATE TABLE settings( First_Name VARCHAR(40), Last_Name VARCHAR(40) );";
		queries[1] = "CREATE TABLE entries( Date DATE, Entry MEDIUMTEXT, PRIMARY KEY( Date ) );";

		for( int i = 0; i <= 1; i++ )
		{
			if( !db.query( queries[i] ) )
			{
				error = db.lastError();
				return false;
			}
		}

		db.disconnect();

		return true;
	}

	std::string JournalFile::_getNewTempFile()
	{
		std::stringstream tmpStream;
		std::string tempFile;

		tmpStream << ".tmp___gtkj___" << rand() << rand();
		tempFile = tmpStream.str();
		tempFile = Globals::TEMP_DIR + tempFile;

		return tempFile;
	}

	void JournalFile::_preferences( bool saving )
	{
		sqlitem::SQliteManager sql = window->sql;
		if( saving && !Globals::plainTextPassword )
		{
			Gtk::MessageDialog dialog( *window, "Since this is a new Journal, you must set your name, username, and password before you can save.",
				Gtk::MESSAGE_INFO );
			dialog.run();
			dialog.hide();
		}

		// select the information from the settings table
		sqlitem::SQliteResult *result = sql.query( "SELECT First_Name, Last_Name FROM settings" );

		if( !result )
		{
			window->_sqlError( sql.lastError() );
		}
		else {

			std::map<std::string, std::string> row;
			if( result->hasNext() ) row = result->next();
			result->finalize();

			// display the informationi dialog and populate it with
			// the information from the SQL query
			aolib::InformationDialog information;
			information.setFirstName( row["First_Name"] );
			information.setLastName( row["Last_Name"] );
			if( Globals::username ) information.setUsername( *Globals::username );
			if( Globals::plainTextPassword ) information.setPassword( *Globals::plainTextPassword );
			information.show();
			int res = information.run();

			if( res == Gtk::RESPONSE_OK )
			{
				// delete the old settings
				sql.go( "DELETE FROM settings;" );

				// insert the new settings
				std::string query = "INSERT INTO settings VALUES( \"" + information.getFirstName()
					+ "\", \"" + information.getLastName() + "\" );";

				if( sql.go( query.c_str() ) == -1 )
				{
					window->_sqlError( sql.lastError() );
				}

				// delete the old global username and password
				if( Globals::plainTextPassword ) delete Globals::plainTextPassword;
				if( Globals::username ) delete Globals::username;

				// create new globals with the new username and password
				Globals::plainTextPassword = new std::string( information.getPassword() );
				Globals::username = new std::string( information.getUsername() );

				// save the database so it matches the password
				if( saving ) window->_saveHandler();
				else {
					window->_journalEntryChangeHandler();
				}
			}
		}
	}
}
